@@ -1,14 +1,14 @@
 package Twitter::API::Error;
 # ABSTRACT: Twitter API exception
-$Twitter::API::Error::VERSION = '0.0108';
+$Twitter::API::Error::VERSION = '0.0109';
 use Moo;
+use List::Util qw/any/;
 use Try::Tiny;
-use Devel::StackTrace;
 use namespace::clean;
 
 use overload '""' => sub { shift->error };
 
-with 'Throwable';
+with qw/Throwable StackTrace::Auto/;
 
 #pod =method http_request
 #pod
@@ -48,27 +48,13 @@ has context => (
 #pod
 #pod =cut
 
-has stack_trace => (
-    is       => 'ro',
-    init_arg => undef,
-    builder  => '_build_stack_trace',
+has '+stack_trace' => (
     handles => {
         stack_frame      => 'frame',
         next_stack_frame => 'next_frame',
     },
 );
 
-sub _build_stack_trace {
-    my $seen;
-    my $this_sub = (caller 0)[3];
-    Devel::StackTrace->new(frame_filter => sub {
-        my $caller = shift->{caller};
-        my $skip = $caller->[0] =~ /^(?:Twitter::API|Throwable|Role::Tiny)\b/
-            || $caller->[3] eq $this_sub;
-        ($seen ||= $skip) && !$skip || 0;
-    });
-}
-
 #pod =method error
 #pod
 #pod Returns a reasonable string representation of the exception. If Twitter
@@ -135,12 +121,14 @@ sub twitter_error_text {
 sub twitter_error_code {
     my $self = shift;
 
-    return $self->twitter_error
-        && exists $self->twitter_error->{errors}
-        && exists $self->twitter_error->{errors}[0]
-        && exists $self->twitter_error->{errors}[0]{code}
-        && $self->twitter_error->{errors}[0]{code}
-        || 0;
+    for ( $self->twitter_error ) {
+        return ref $_ eq 'HASH'
+            && exists $_->{errors}
+            && exists $_->{errors}[0]
+            && exists $_->{errors}[0]{code}
+            && $_->{errors}[0]{code}
+            || 0;
+    }
 }
 
 #pod =method is_token_error
@@ -154,28 +142,39 @@ sub twitter_error_code {
 #pod indicate a problem with the resource requested. For example, the target account
 #pod no longer exists.
 #pod
+#pod is_token_error returns true for the following Twitter API errors:
+#pod
+#pod =for :list
+#pod * 32:  could not authenticate you
+#pod * 64:  this account is suspended
+#pod * 88:  rate limit exceeded for this token
+#pod * 89:  invalid or expired tokens
+#pod * 215: Bad authentication data
+#pod * 226: This request looks like it might be automated. To protect our users from
+#pod   spam and other malicious activity, we can’t complete this action right now.
+#pod * 326: To protect our users from spam…
+#pod
+#pod For error 215, Twitter's API documentation says, "Typically sent with 1.1
+#pod responses with HTTP code 400. The method requires authentication but it was not
+#pod presented or was wholly invalid." In practice, though, this error seems to be
+#pod spurious, and often succeeds if retried, even with the same tokens.
+#pod
+#pod The Twitter API documentation describes error code 226, but in practice, they
+#pod use code 326 instead, so we check for both. This error code means the account
+#pod the tokens belong to has been locked for spam like activity and can't be used
+#pod by the API until the user takes action to unlock their account.
+#pod
+#pod See Twitter's L<Error Codes &
+#pod Responses|https://dev.twitter.com/overview/api/response-codes> documentation
+#pod for more information.
+#pod
 #pod =cut
 
-# Some twitter errors result from issues with the target of a call. Others,
-# from an issue with the tokens used to make the call. In the latter case, we
-# should just retry with fresh tokens. Return true for the latter type.
-sub is_token_error {
-    my $self = shift;
+use constant TOKEN_ERRORS => ( 32, 64, 88, 89, 215, 226, 326 );
 
-    # 32:  could not authenticate you
-    # 64:  this account is suspended
-    # 88:  rate limit exceeded for this token
-    # 89:  invalid or expired tokens
-    #
-    # Twitter documents error 226 as spam/bot-like behavior,
-    # but they actually send 326! So, we'll look for both.
-    # 226: this account locked for bot-like behavior
-    # 326: To protect our users from spam...
-    my $code = $self->twitter_error_code;
-    for ( 32, 64, 88, 89, 226, 326 ) {
-        return 1 if $_ == $code;
-    }
-    return 0;
+sub is_token_error {
+    my $code = shift->twitter_error_code;
+    return any { $code == $_ } TOKEN_ERRORS;
 }
 
 #pod =method http_response_code
@@ -222,7 +221,7 @@ Twitter::API::Error - Twitter API exception
 
 =head1 VERSION
 
-version 0.0108
+version 0.0109
 
 =head1 SYNOPSIS
 
@@ -236,7 +235,7 @@ version 0.0108
         my $r = $client->get('account/verify_credentials');
     }
     catch {
-        die $_ unless _is_twitter_api_error;
+        die $_ unless is_twitter_api_error;
 
         warn "Twitter says: ", $_->twitter_error_text;
     };
@@ -299,6 +298,54 @@ suspended or access to the application revoked by the user. Other error codes
 indicate a problem with the resource requested. For example, the target account
 no longer exists.
 
+is_token_error returns true for the following Twitter API errors:
+
+=over 4
+
+=item *
+
+32:  could not authenticate you
+
+=item *
+
+64:  this account is suspended
+
+=item *
+
+88:  rate limit exceeded for this token
+
+=item *
+
+89:  invalid or expired tokens
+
+=item *
+
+215: Bad authentication data
+
+=item *
+
+226: This request looks like it might be automated. To protect our users from spam and other malicious activity, we can’t complete this action right now.
+
+=item *
+
+326: To protect our users from spam…
+
+=back
+
+For error 215, Twitter's API documentation says, "Typically sent with 1.1
+responses with HTTP code 400. The method requires authentication but it was not
+presented or was wholly invalid." In practice, though, this error seems to be
+spurious, and often succeeds if retried, even with the same tokens.
+
+The Twitter API documentation describes error code 226, but in practice, they
+use code 326 instead, so we check for both. This error code means the account
+the tokens belong to has been locked for spam like activity and can't be used
+by the API until the user takes action to unlock their account.
+
+See Twitter's L<Error Codes &
+Responses|https://dev.twitter.com/overview/api/response-codes> documentation
+for more information.
+
 =head2 http_response_code
 
 Delegates to C<<http_response->code>>. Returns the HTTP status code of the
